@@ -5,12 +5,16 @@ import tempfile
 from datetime import datetime, timedelta
 from pathlib import Path
 from typing import Dict, List, Optional, Union
+
+from numpy import isin
 from feathr.definition.feature import FeatureBase
 
 import redis
 from azure.identity import DefaultAzureCredential
 from jinja2 import Template
 from pyhocon import ConfigFactory
+from feathr.definition.sink import GenericSink, Sink
+from feathr.definition.source import GenericSource, Source
 from feathr.registry.feature_registry import default_registry_client
 
 from feathr.spark_provider._databricks_submission import _FeathrDatabricksJobLauncher
@@ -44,11 +48,26 @@ class FeatureJoinJobParams:
         job_output_path: Absolute path in Cloud that you want your output data to be in.
     """
 
-    def __init__(self, join_config_path, observation_path, feature_config, job_output_path):
+    def __init__(self, join_config_path, observation_path, feature_config, job_output_path, secrets:List[str]=[]):
+        self.secrets = secrets
         self.join_config_path = join_config_path
-        self.observation_path = observation_path
+        if isinstance(observation_path, str):
+            self.observation_path = observation_path
+        elif isinstance(observation_path, Source):
+            self.observation_path = observation_path.to_argument()
+            if hasattr(observation_path, "get_required_properties"):
+                self.secrets.extend(observation_path.get_required_properties())
+        else:
+            raise TypeError("observation_path must be a string or a Sink")
         self.feature_config = feature_config
-        self.job_output_path = job_output_path
+        if isinstance(job_output_path, str):
+            self.job_output_path = job_output_path
+        elif isinstance(job_output_path, Sink):
+            self.job_output_path = job_output_path.to_argument()
+            if hasattr(job_output_path, "get_required_properties"):
+                self.secrets.extend(job_output_path.get_required_properties())
+        else:
+            raise TypeError("job_output_path must be a string or a Sink")
 
 
 class FeatureGenerationJobParams:
@@ -57,11 +76,13 @@ class FeatureGenerationJobParams:
     Attributes:
         generation_config_path: Path to the feature generation config.
         feature_config: Path to the features config.
+        secrets: secret names from data sources, the values will be taken from env or KeyVault
     """
 
-    def __init__(self, generation_config_path, feature_config):
+    def __init__(self, generation_config_path, feature_config, secrets=[]):
         self.generation_config_path = generation_config_path
         self.feature_config = feature_config
+        self.secrets = secrets
 
 
 
@@ -115,9 +136,24 @@ class FeathrClient(object):
         self.redis_ssl_enabled = self.envutils.get_environment_variable_with_default(
             'online_store', 'redis', 'ssl_enabled')
 
+        # Offline store enabled configs; false by default
+        self.s3_enabled = self.envutils.get_environment_variable_with_default(
+            'offline_store', 's3', 's3_enabled')
+        self.adls_enabled = self.envutils.get_environment_variable_with_default(
+            'offline_store', 'adls', 'adls_enabled')
+        self.wasb_enabled = self.envutils.get_environment_variable_with_default(
+            'offline_store', 'wasb', 'wasb_enabled')
+        self.jdbc_enabled = self.envutils.get_environment_variable_with_default(
+            'offline_store', 'jdbc', 'jdbc_enabled')
+        self.snowflake_enabled = self.envutils.get_environment_variable_with_default(
+            'offline_store', 'snowflake', 'snowflake_enabled')
+        if not (self.s3_enabled or self.adls_enabled or self.wasb_enabled or self.jdbc_enabled or self.snowflake_enabled):
+            self.logger.warning("No offline storage enabled.")
+
         # S3 configs
-        self.s3_endpoint = self.envutils.get_environment_variable_with_default(
-            'offline_store', 's3', 's3_endpoint')
+        if self.s3_enabled:
+            self.s3_endpoint = self.envutils.get_environment_variable_with_default(
+                'offline_store', 's3', 's3_endpoint')
 
         # spark configs
         self.output_num_parts = self.envutils.get_environment_variable_with_default(
@@ -174,6 +210,7 @@ class FeathrClient(object):
 
         self._construct_redis_client()
 
+        self.secret_names = []
 
         # initialize registry
         self.registry = default_registry_client(self.project_name, config_path=config_path, credential=self.credential)
@@ -183,9 +220,7 @@ class FeathrClient(object):
 
         Some required information has to be set via environment variables so the client can work.
         """
-        props = []
-        if hasattr(self, "system_properties"):
-            props = self.system_properties
+        props = self.secret_names
         for required_field in (self.required_fields + props):
             if required_field not in os.environ:
                 raise RuntimeError(f'{required_field} is not set in environment variable. All required environment '
@@ -240,12 +275,12 @@ class FeathrClient(object):
         for anchor in self.anchor_list:
             if hasattr(anchor.source, "get_required_properties"):
                 props.extend(anchor.source.get_required_properties())
-        if len(props)>0:
-            self.system_properties = props
+        # Reset `system_properties`
+        self.secret_names = props
 
         # Pretty print anchor_list
         if verbose and self.anchor_list:
-                FeaturePrinter.pretty_print_anchors(self.anchor_list)
+            FeaturePrinter.pretty_print_anchors(self.anchor_list)
 
     def list_registered_features(self, project_name: str = None) -> List[str]:
         """List all the already registered features under the given project.
@@ -421,7 +456,7 @@ class FeathrClient(object):
     def get_offline_features(self,
                              observation_settings: ObservationSettings,
                              feature_query: Union[FeatureQuery, List[FeatureQuery]],
-                             output_path: str,
+                             output_path: Union[str, Sink],
                              execution_configurations: Union[SparkExecutionConfiguration ,Dict[str,str]] = {},
                              udf_files = None,
                              verbose: bool = False
@@ -469,9 +504,16 @@ class FeathrClient(object):
             FeaturePrinter.pretty_print_feature_query(feature_query)
 
         write_to_file(content=config, full_file_name=config_file_path)
-        return self._get_offline_features_with_config(config_file_path, execution_configurations, udf_files=udf_files)
+        return self._get_offline_features_with_config(config_file_path,
+                                                      output_path=output_path,
+                                                      execution_configurations=execution_configurations,
+                                                      udf_files=udf_files)
 
-    def _get_offline_features_with_config(self, feature_join_conf_path='feature_join_conf/feature_join.conf', execution_configurations: Dict[str,str] = {}, udf_files=[]):
+    def _get_offline_features_with_config(self,
+                                          feature_join_conf_path='feature_join_conf/feature_join.conf',
+                                          output_path: Union[str, Sink] = "",
+                                          execution_configurations: Dict[str,str] = {},
+                                          udf_files=[]):
         """Joins the features to your offline observation dataset based on the join config.
 
         Args:
@@ -483,8 +525,7 @@ class FeathrClient(object):
         feature_join_job_params = FeatureJoinJobParams(join_config_path=os.path.abspath(feature_join_conf_path),
                                                        observation_path=feathr_feature['observationPath'],
                                                        feature_config=os.path.join(self.local_workspace_dir, 'feature_conf/'),
-                                                       job_output_path=feathr_feature['outputPath'],
-                                                       )
+                                                       job_output_path=output_path)
         job_tags = {OUTPUT_PATH_TAG:feature_join_job_params.job_output_path}
         # set output format in job tags if it's set by user, so that it can be used to parse the job result in the helper function
         if execution_configurations is not None and OUTPUT_FORMAT in execution_configurations:
@@ -502,25 +543,39 @@ class FeathrClient(object):
             python_files=cloud_udf_paths,
             job_tags=job_tags,
             main_class_name='com.linkedin.feathr.offline.job.FeatureJoinJob',
-            arguments=[
+            arguments= [
                 '--join-config', self.feathr_spark_launcher.upload_or_get_cloud_path(
                     feature_join_job_params.join_config_path),
                 '--input', feature_join_job_params.observation_path,
                 '--output', feature_join_job_params.job_output_path,
                 '--feature-config', self.feathr_spark_launcher.upload_or_get_cloud_path(
                     feature_join_job_params.feature_config),
-                '--num-parts', self.output_num_parts,
-                '--s3-config', self._get_s3_config_str(),
-                '--adls-config', self._get_adls_config_str(),
-                '--blob-config', self._get_blob_config_str(),
-                '--sql-config', self._get_sql_config_str(),
-                '--snowflake-config', self._get_snowflake_config_str()
-            ],
+                '--num-parts', self.output_num_parts
+            ]+self._get_offline_storage_arguments(),
             reference_files_path=[],
             configuration=execution_configurations,
-            properties=self._get_system_properties()
+            properties=self._collect_secrets(feature_join_job_params.secrets)
         )
 
+    def _get_offline_storage_arguments(self):
+        arguments = []
+        if self.s3_enabled:
+            arguments.append('--s3-config')
+            arguments.append(self._get_s3_config_str())
+        if self.adls_enabled:
+            arguments.append('--adls-config')
+            arguments.append(self._get_adls_config_str())
+        if self.wasb_enabled:
+            arguments.append('--blob-config')
+            arguments.append(self._get_blob_config_str())
+        if self.jdbc_enabled:
+            arguments.append('--sql-config')
+            arguments.append(self._get_sql_config_str())
+        if self.snowflake_enabled:
+            arguments.append('--snowflake-config')
+            arguments.append(self._get_snowflake_config_str())
+        return arguments
+
     def get_job_result_uri(self, block=True, timeout_sec=300) -> str:
         """Gets the job output URI
         """
@@ -562,6 +617,13 @@ class FeathrClient(object):
             settings: Feature materialization settings
             execution_configurations: a dict that will be passed to spark job when the job starts up, i.e. the "spark configurations". Note that not all of the configuration will be honored since some of the configurations are managed by the Spark platform, such as Databricks or Azure Synapse. Refer to the [spark documentation](https://spark.apache.org/docs/latest/configuration.html) for a complete list of spark configurations.
         """
+        
+        # Collect secrets from sinks
+        secrets = []
+        for sink in settings.sinks:
+            if hasattr(sink, "get_required_properties"):
+                secrets.extend(sink.get_required_properties())
+        
         # produce materialization config
         for end in settings.get_backfill_cutoff_time():
             settings.backfill_time.end = end
@@ -579,8 +641,8 @@ class FeathrClient(object):
                 raise RuntimeError("Please call FeathrClient.build_features() first in order to materialize the features")
 
             udf_files = _PreprocessingPyudfManager.prepare_pyspark_udf_files(settings.feature_names, self.local_workspace_dir)
-            # CLI will directly call this so the experiene won't be broken
-            self._materialize_features_with_config(config_file_path, execution_configurations, udf_files)
+            # CLI will directly call this so the experience won't be broken
+            self._materialize_features_with_config(config_file_path, execution_configurations, udf_files, secrets)
             if os.path.exists(config_file_path):
                 os.remove(config_file_path)
 
@@ -588,7 +650,7 @@ class FeathrClient(object):
         if verbose and settings:
             FeaturePrinter.pretty_print_materialize_features(settings)
 
-    def _materialize_features_with_config(self, feature_gen_conf_path: str = 'feature_gen_conf/feature_gen.conf',execution_configurations: Dict[str,str] = {}, udf_files=[]):
+    def _materialize_features_with_config(self, feature_gen_conf_path: str = 'feature_gen_conf/feature_gen.conf',execution_configurations: Dict[str,str] = {}, udf_files=[], secrets=[]):
         """Materializes feature data based on the feature generation config. The feature
         data will be materialized to the destination specified in the feature generation config.
 
@@ -617,12 +679,7 @@ class FeathrClient(object):
                 '--feature-config', self.feathr_spark_launcher.upload_or_get_cloud_path(
                 generation_config.feature_config),
                 '--redis-config', self._getRedisConfigStr(),
-                '--s3-config', self._get_s3_config_str(),
-                '--adls-config', self._get_adls_config_str(),
-                '--blob-config', self._get_blob_config_str(),
-                '--sql-config', self._get_sql_config_str(),
-                '--snowflake-config', self._get_snowflake_config_str(),
-            ] + optional_params
+            ] + self._get_offline_storage_arguments()+optional_params
         monitoring_config_str = self._get_monitoring_config_str()
         if monitoring_config_str:
             arguments.append('--monitoring-config')
@@ -635,7 +692,7 @@ class FeathrClient(object):
             arguments=arguments,
             reference_files_path=[],
             configuration=execution_configurations,
-            properties=self._get_system_properties()
+            properties=self._collect_secrets(secrets)
         )
 
 
@@ -768,14 +825,13 @@ class FeathrClient(object):
             """.format(sasl=sasl)
         return config_str
 
-    def _get_system_properties(self):
-        """Go through all data sources and fill all required system properties"""
+    def _collect_secrets(self, additional_secrets=[]):
+        """Collect all values corresponding to the secret names."""
         prop_and_value = {}
-        if hasattr(self, "system_properties"):
-            for prop in self.system_properties:
-                prop_and_value[prop] = self.envutils.get_environment_variable_with_default(prop)
-            return prop_and_value
-        return None
+        for prop in self.secret_names + additional_secrets:
+            prop = prop.upper()
+            prop_and_value[prop] = self.envutils.get_environment_variable_with_default(prop)
+        return prop_and_value
 
     def get_features_from_registry(self, project_name: str) -> Dict[str, FeatureBase]:
         """
