@@ -7,7 +7,7 @@ import distutils.dir_util
 import subprocess
 import urllib.request
 from feathr.client import FeathrClient
-
+from feathr.registry._feathr_registry_client import _FeatureRegistry
 
 @click.group()
 @click.pass_context
@@ -71,6 +71,18 @@ def init(name, git):
     click.echo(click.style('Feathr initialization completed.', fg='green'))
 
 
+@cli.command()
+@click.option('--save_to', default="./", help='Specify the path to save the output HOCON config(relative to current path).')
+def hocon(save_to):
+    """
+    Scan all Python-based feature definitions recursively under current directory,
+     convert them to HOCON config and save to a given path (relative to current directory).
+    """
+    scan_dir = Path.cwd()
+    save_to = Path(os.path.join(scan_dir, save_to))
+    _FeatureRegistry.save_to_feature_config(scan_dir, save_to)
+
+
 @cli.command()
 @click.argument('filepath', default='feature_join_conf/feature_join.conf', type=click.Path(exists=True))
 def join(filepath):
