@@ -68,7 +68,8 @@ def basic_test_setup(config_path: str):
                             feature_type=FLOAT,
                             transform=WindowAggTransformation(agg_expr="cast_float(fare_amount)",
                                                               agg_func="AVG",
-                                                              window="90d")),
+                                                              window="90d",
+                                                              )),
                     Feature(name="f_location_max_fare",
                             key=location_id,
                             feature_type=FLOAT,
@@ -194,6 +195,7 @@ def registry_test_setup_append(config_path: str):
 
 def generate_entities():
     def add_new_dropoff_and_fare_amount_column(df: DataFrame):
+        from pyspark.sql.functions import col
         df = df.withColumn("new_lpep_dropoff_datetime", col("lpep_dropoff_datetime"))
         df = df.withColumn("new_fare_amount", col("fare_amount") + 1000000)
         return df
@@ -267,7 +269,7 @@ def generate_entities():
     derived_feature_list = [
                         f_trip_time_distance, f_trip_time_rounded, f_trip_time_rounded_plus]
     
-    # shuffule the order to make sure they can be parsed correctly
+    # shuffle the order to make sure they can be parsed correctly
     # Those input derived features can be in arbitrary order, but in order to parse the right dependencies, we need to reorder them internally in a certain order. 
     # This shuffle is to make sure that each time we have random shuffle for the input and make sure the internal sorting algorithm works (we are using topological sort).
     random.shuffle(derived_feature_list)
@@ -283,6 +285,7 @@ def registry_test_setup_append(config_path: str):
     client = FeathrClient(config_path=config_path, project_registry_tag={"for_test_purpose":"true"})
 
     def add_new_dropoff_and_fare_amount_column(df: DataFrame):
+        from pyspark.sql.functions import col
         df = df.withColumn("new_lpep_dropoff_datetime", col("lpep_dropoff_datetime"))
         df = df.withColumn("new_fare_amount", col("fare_amount") + 1000000)
         return df
@@ -362,7 +365,7 @@ def registry_test_setup_append(config_path: str):
     derived_feature_list = [
                         f_trip_time_distance, f_trip_time_rounded, f_trip_time_rounded_plus]
     
-    # shuffule the order to make sure they can be parsed correctly
+    # shuffle the order to make sure they can be parsed correctly
     # Those input derived features can be in arbitrary order, but in order to parse the right dependencies, we need to reorder them internally in a certain order. 
     # This shuffle is to make sure that each time we have random shuffle for the input and make sure the internal sorting algorithm works (we are using topological sort).
     random.shuffle(derived_feature_list)
@@ -372,4 +375,6 @@ def registry_test_setup_append(config_path: str):
 def get_online_test_table_name(table_name: str):
     # use different time for testing to avoid write conflicts
     now = datetime.now()
-    return '_'.join([table_name, str(now.minute), str(now.second)])
\ No newline at end of file
+    res_table = '_'.join([table_name, str(now.minute), str(now.second)])
+    print("The online Redis table is", res_table)
+    return res_table
\ No newline at end of file
